package com.albertoventurini.parsley.grammar;

import com.albertoventurini.parsley.grammar.rules.Rule;

import java.util.function.Predicate;

public class GrammarContext extends ParseContext {
    private final Rule commentRule;
    private final Predicate<Character> whitespacePredicate;
    private boolean inComment;

    GrammarContext(
            final String string,
            final Rule commentRule,
            final Predicate<Character> whitespacePredicate) {
        super(string);
        this.commentRule = commentRule;
        this.whitespacePredicate = whitespacePredicate;
    }

    public void advanceToNextToken() {
        discardWhitespaces();
        if (!inComment) {
            discardComments();
            discardWhitespaces();
        }
    }

    public void advanceToEndOfToken() {
        while (hasNext()) {
            if (commentRule.apply(this).isPresent()) {
                break;
            }

            final char c = peek();
            if (isWhitespace(c)) {
                break;
            }
        }
    }

    public boolean isInComment() {
        return inComment;
    }

    public void setInComment(final boolean inComment) {
        this.inComment = inComment;
    }

    private void discardComments() {
        if (hasNext() && commentRule != null) {
            commentRule.apply(this);
        }
    }

    private void discardWhitespaces() {
        while (hasNext()) {
            final char c = peek();
            if (!isWhitespace(c)) {
                break;
            }
            advance();
        }
    }

    private boolean isWhitespace(final char c) {
        return whitespacePredicate.test(c);
//        return Character.isWhitespace(c) || c == '\n';
//        return c != '\n' && Character.isWhitespace(c);
    }
}
